@@ -1,15 +1,15 @@
 /* Copyright (C) 2000 MySQL AB & MySQL Finland AB & TCX DataKonsult AB
-   
+
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
-   
+
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
-   
+
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
@@ -20,6 +20,7 @@
 #include "mysql_priv.h"
 #include <my_dir.h>
 #include <m_ctype.h>
+#include "sql_repl.h"
 
 class READ_INFO {
   File	file;
@@ -32,6 +33,7 @@ class READ_INFO {
   int	field_term_char,line_term_char,enclosed_char,escape_char;
   int	*stack,*stack_pos;
   bool	found_end_of_line,start_of_line,eof;
+  bool  need_end_io_cache;
   IO_CACHE cache;
   NET *io_net;
 
@@ -50,6 +52,22 @@ public:
   char unescape(char chr);
   int terminator(char *ptr,uint length);
   bool find_start_of_fields();
+  /*
+    We need to force cache close before destructor is invoked to log
+    the last read block
+  */
+  void end_io_cache()
+  {
+    ::end_io_cache(&cache);
+    need_end_io_cache = 0;
+  }
+
+  /*
+    Either this method, or we need to make cache public
+    Arg must be set from mysql_load() since constructor does not see
+    either the table or THD value
+  */
+  void set_io_cache_arg(void* arg) { cache.arg = arg; }
 };
 
 static int read_fixed_length(THD *thd,COPY_INFO &info,TABLE *table,
@@ -67,23 +85,31 @@ int mysql_load(THD *thd,sql_exchange *ex,TABLE_LIST *table_list,
   File file;
   TABLE *table;
   int error;
-  uint save_skip_lines = ex->skip_lines;
   String *field_term=ex->field_term,*escaped=ex->escaped,
     *enclosed=ex->enclosed;
   bool is_fifo=0;
-  bool using_transactions;
-
+  LOAD_FILE_INFO lf_info;
+  char *db = table_list->db;			// This is never null
+  /* If no current database, use database where table is located */
+  char *tdb= thd->db ? thd->db : db;
+  bool transactional_table, log_delayed;
   DBUG_ENTER("mysql_load");
 
+#ifdef EMBEDDED_LIBRARY
+  read_file_from_client  = 0; //server is always in the same process 
+#endif
+
   if (escaped->length() > 1 || enclosed->length() > 1)
   {
     my_message(ER_WRONG_FIELD_TERMINATORS,ER(ER_WRONG_FIELD_TERMINATORS),
 	       MYF(0));
     DBUG_RETURN(-1);
   }
-
   if (!(table = open_ltable(thd,table_list,lock_type)))
     DBUG_RETURN(-1);
+  transactional_table= table->file->has_transactions();
+  log_delayed= (transactional_table || table->tmp_table);
+
   if (!fields.elements)
   {
     Field **field;
@@ -93,7 +119,7 @@ int mysql_load(THD *thd,sql_exchange *ex,TABLE_LIST *table_list,
   else
   {						// Part field list
     thd->dupp_field=0;
-    if (setup_tables(table_list) || setup_fields(thd,table_list,fields,1,0))
+    if (setup_tables(table_list) || setup_fields(thd,table_list,fields,1,0,0))
       DBUG_RETURN(-1);
     if (thd->dupp_field)
     {
@@ -104,7 +130,7 @@ int mysql_load(THD *thd,sql_exchange *ex,TABLE_LIST *table_list,
 
   uint tot_length=0;
   bool use_blobs=0,use_timestamp=0;
-  List_iterator<Item> it(fields);
+  List_iterator_fast<Item> it(fields);
 
   Item_field *field;
   while ((field=(Item_field*) it++))
@@ -134,37 +160,32 @@ int mysql_load(THD *thd,sql_exchange *ex,TABLE_LIST *table_list,
 
   if (read_file_from_client)
   {
-    char tmp [FN_REFLEN+1],*end;
-    DBUG_PRINT("info",("reading local file"));
-    tmp[0] = (char) 251;				/* NULL_LENGTH */
-    end=strnmov(tmp+1,ex->file_name,sizeof(tmp)-2);
-    (void) my_net_write(&thd->net,tmp,(uint) (end-tmp));
-    (void) net_flush(&thd->net);
+    (void)net_request_file(&thd->net,ex->file_name);
     file = -1;
   }
   else
   {
-    read_file_from_client=0;
 #ifdef DONT_ALLOW_FULL_LOAD_DATA_PATHS
     ex->file_name+=dirname_length(ex->file_name);
 #endif
     if (!dirname_length(ex->file_name) &&
-	strlen(ex->file_name)+strlen(mysql_data_home)+strlen(thd->db)+3 <
+	strlen(ex->file_name)+strlen(mysql_data_home)+strlen(tdb)+3 <
 	FN_REFLEN)
     {
-      (void) sprintf(name,"%s/%s/%s",mysql_data_home,thd->db,ex->file_name);
+      (void) sprintf(name,"%s/%s/%s",mysql_data_home,tdb,ex->file_name);
       unpack_filename(name,name);		/* Convert to system format */
     }
     else
     {
       unpack_filename(name,ex->file_name);
-#if !defined(__WIN__) && !defined(OS2)
+#if !defined(__WIN__) && !defined(OS2) && ! defined(__NETWARE__)
       MY_STAT stat_info;
       if (!my_stat(name,&stat_info,MYF(MY_WME)))
 	DBUG_RETURN(-1);
-      
-      // the file must be:
-      if (!((stat_info.st_mode & S_IROTH) == S_IROTH &&  // readable by others
+
+      // if we are not in slave thread, the file must be:
+      if (!thd->slave_thread &&
+	  !((stat_info.st_mode & S_IROTH) == S_IROTH &&  // readable by others
 #ifndef __EMX__
 	    (stat_info.st_mode & S_IFLNK) != S_IFLNK && // and not a symlink
 #endif
@@ -197,13 +218,28 @@ int mysql_load(THD *thd,sql_exchange *ex,TABLE_LIST *table_list,
     DBUG_RETURN(-1);				// Can't allocate buffers
   }
 
+  if (!opt_old_rpl_compat && mysql_bin_log.is_open())
+  {
+    lf_info.thd = thd;
+    lf_info.ex = ex;
+    lf_info.db = db;
+    lf_info.table_name = table_list->real_name;
+    lf_info.fields = &fields;
+    lf_info.handle_dup = handle_duplicates;
+    lf_info.wrote_create_file = 0;
+    lf_info.last_pos_in_file = HA_POS_ERROR;
+    lf_info.log_delayed= log_delayed;
+    read_info.set_io_cache_arg((void*) &lf_info);
+  }
   restore_record(table,2);
 
   thd->count_cuted_fields=1;			/* calc cuted fields */
   thd->cuted_fields=0L;
   if (ex->line_term->length() && field_term->length())
   {
-    while (ex->skip_lines--)
+    // ex->skip_lines needs to be preserved for logging
+    uint skip_lines = ex->skip_lines;
+    while (skip_lines--)
     {
       if (read_info.next_line())
 	break;
@@ -215,7 +251,9 @@ int mysql_load(THD *thd,sql_exchange *ex,TABLE_LIST *table_list,
     if (use_timestamp)
       table->time_stamp=0;
     table->next_number_field=table->found_next_number_field;
-    VOID(table->file->extra(HA_EXTRA_WRITE_CACHE));
+    VOID(table->file->extra_opt(HA_EXTRA_WRITE_CACHE,
+			    thd->variables.read_buff_size));
+    table->bulk_insert= 1;
     if (handle_duplicates == DUP_IGNORE ||
 	handle_duplicates == DUP_REPLACE)
       table->file->extra(HA_EXTRA_IGNORE_DUP_KEY);
@@ -225,47 +263,102 @@ int mysql_load(THD *thd,sql_exchange *ex,TABLE_LIST *table_list,
       error=read_fixed_length(thd,info,table,fields,read_info);
     else
       error=read_sep_field(thd,info,table,fields,read_info,*enclosed);
-    if (table->file->extra(HA_EXTRA_NO_CACHE) ||
-	table->file->activate_all_index(thd))
-      error=1; /* purecov: inspected */
+    if (table->file->extra(HA_EXTRA_NO_CACHE))
+      error=1;					/* purecov: inspected */
+    if (table->file->activate_all_index(thd))
+      error=1;					/* purecov: inspected */
     table->file->extra(HA_EXTRA_NO_IGNORE_DUP_KEY);
     table->time_stamp=save_time_stamp;
     table->next_number_field=0;
-    if (thd->lock)
-    {
-      mysql_unlock_tables(thd, thd->lock);
-      thd->lock=0;
-    }
   }
   if  (file >= 0) my_close(file,MYF(0));
   free_blobs(table);				/* if pack_blob was used */
   table->copy_blobs=0;
   thd->count_cuted_fields=0;			/* Don`t calc cuted fields */
-  using_transactions = table->file->has_transactions();
+
   if (error)
   {
-    if (using_transactions)
+    if (transactional_table)
       ha_autocommit_or_rollback(thd,error);
-    DBUG_RETURN(-1);                           // Error on read
+    if (!opt_old_rpl_compat && mysql_bin_log.is_open())
+    {
+      /*
+        Make sure last block (the one which caused the error) gets logged.
+        This is needed because otherwise after write of
+        (to the binlog, not to read_info (which is a cache))
+        Delete_file_log_event the bad block will remain in read_info (because
+        pre_read is not called at the end of the last block; remember pre_read
+        is called whenever a new block is read from disk).
+        At the end of mysql_load(), the destructor of read_info will call
+        end_io_cache() which will flush read_info, so we will finally have
+        this in the binlog:
+        Append_block # The last successfull block
+        Delete_file
+        Append_block # The failing block
+        which is nonsense.
+        Or could also be (for a small file)
+        Create_file  # The failing block
+        which is nonsense (Delete_file is not written in this case, because:
+        Create_file has not been written, so Delete_file is not written, then
+        when read_info is destroyed end_io_cache() is called which writes
+        Create_file.
+      */
+      read_info.end_io_cache();
+      /* If the file was not empty, wrote_create_file is true */
+      if (lf_info.wrote_create_file)
+      {
+        Delete_file_log_event d(thd, db, log_delayed);
+        mysql_bin_log.write(&d);
+      }
+    }
+    error= -1;				// Error on read
+    goto err;
   }
   sprintf(name,ER(ER_LOAD_INFO),info.records,info.deleted,
 	  info.records-info.copied,thd->cuted_fields);
   send_ok(&thd->net,info.copied+info.deleted,0L,name);
   // on the slave thd->query is never initialized
-  if(!thd->slave_thread)
+  if (!thd->slave_thread)
     mysql_update_log.write(thd,thd->query,thd->query_length);
-  
-  if (!using_transactions)
+
+  if (!log_delayed)
     thd->options|=OPTION_STATUS_NO_TRANS_UPDATE;
-  if (!read_file_from_client && mysql_bin_log.is_open())
+  if (mysql_bin_log.is_open())
   {
-    ex->skip_lines = save_skip_lines; 
-    Load_log_event qinfo(thd, ex, table->table_name, fields, 
-			 handle_duplicates);
-    mysql_bin_log.write(&qinfo);
+    if (opt_old_rpl_compat)
+    {
+      if (!read_file_from_client)
+      {
+	Load_log_event qinfo(thd, ex, db, table->table_name, fields, 
+			     handle_duplicates, log_delayed);
+	mysql_bin_log.write(&qinfo);
+      }
+    }
+    else
+    {
+      /*
+        As already explained above, we need to call end_io_cache() or the last
+        block will be logged only after Execute_load_log_event (which is wrong),
+        when read_info is destroyed.
+      */
+      read_info.end_io_cache(); 
+      if (lf_info.wrote_create_file)
+      {
+        Execute_load_log_event e(thd, db, log_delayed);
+        mysql_bin_log.write(&e);
+      }
+    }
   }
-  if (using_transactions)
+  if (transactional_table)
     error=ha_autocommit_or_rollback(thd,error); 
+  query_cache_invalidate3(thd, table_list, 0);
+
+err:
+  if (thd->lock)
+  {
+    mysql_unlock_tables(thd, thd->lock);
+    thd->lock=0;
+  }
   DBUG_RETURN(error);
 }
 
@@ -278,10 +371,12 @@ static int
 read_fixed_length(THD *thd,COPY_INFO &info,TABLE *table,List<Item> &fields,
 		  READ_INFO &read_info)
 {
-  List_iterator<Item> it(fields);
+  List_iterator_fast<Item> it(fields);
   Item_field *sql_field;
+  ulonglong id;
   DBUG_ENTER("read_fixed_length");
 
+  id=0;
   /* No fields can be null in this format. mark all fields as not null */
   while ((sql_field= (Item_field*) it++))
       sql_field->field->set_notnull();
@@ -324,13 +419,23 @@ read_fixed_length(THD *thd,COPY_INFO &info,TABLE *table,List<Item> &fields,
       thd->cuted_fields++;			/* To long row */
     if (write_record(table,&info))
       DBUG_RETURN(1);
+    /*
+      If auto_increment values are used, save the first one
+       for LAST_INSERT_ID() and for the binary/update log.
+       We can't use insert_id() as we don't want to touch the
+       last_insert_id_used flag.
+    */
+    if (!id && thd->insert_id_used)
+      id= thd->last_insert_id;
     if (table->next_number_field)
       table->next_number_field->reset();	// Clear for next record
-    if (read_info.next_line())			// Skipp to next line
+    if (read_info.next_line())			// Skip to next line
       break;
     if (read_info.line_cuted)
       thd->cuted_fields++;			/* To long row */
   }
+  if (id && !read_info.error)
+    thd->insert_id(id);			// For binary/update log
   DBUG_RETURN(test(read_info.error));
 }
 
@@ -341,13 +446,15 @@ read_sep_field(THD *thd,COPY_INFO &info,TABLE *table,
 	       List<Item> &fields, READ_INFO &read_info,
 	       String &enclosed)
 {
-  List_iterator<Item> it(fields);
+  List_iterator_fast<Item> it(fields);
   Item_field *sql_field;
   uint enclosed_length;
+  ulonglong id;
   DBUG_ENTER("read_sep_field");
 
   enclosed_length=enclosed.length();
-
+  id=0;
+  
   for (;;it.rewind())
   {
     if (thd->killed)
@@ -376,7 +483,7 @@ read_sep_field(THD *thd,COPY_INFO &info,TABLE *table,
 	{
 	  if (field->type() == FIELD_TYPE_TIMESTAMP)
 	    ((Field_timestamp*) field)->set_time();
-	  else
+	  else if (field != table->next_number_field)
 	    thd->cuted_fields++;
 	}
 	continue;
@@ -391,7 +498,7 @@ read_sep_field(THD *thd,COPY_INFO &info,TABLE *table,
     {						// Last record
       if (sql_field == (Item_field*) fields.head())
 	break;
-      for ( ; sql_field ; sql_field=(Item_field*) it++)
+      for (; sql_field ; sql_field=(Item_field*) it++)
       {
 	sql_field->field->set_null();
 	sql_field->field->reset();
@@ -400,13 +507,23 @@ read_sep_field(THD *thd,COPY_INFO &info,TABLE *table,
     }
     if (write_record(table,&info))
       DBUG_RETURN(1);
+    /*
+      If auto_increment values are used, save the first one
+       for LAST_INSERT_ID() and for the binary/update log.
+       We can't use insert_id() as we don't want to touch the
+       last_insert_id_used flag.
+    */
+    if (!id && thd->insert_id_used)
+      id= thd->last_insert_id;
     if (table->next_number_field)
       table->next_number_field->reset();	// Clear for next record
-    if (read_info.next_line())			// Skipp to next line
+    if (read_info.next_line())			// Skip to next line
       break;
     if (read_info.line_cuted)
       thd->cuted_fields++;			/* To long row */
   }
+  if (id && !read_info.error)
+    thd->insert_id(id);			// For binary/update log
   DBUG_RETURN(test(read_info.error));
 }
 
@@ -431,8 +548,10 @@ READ_INFO::unescape(char chr)
 }
 
 
-	/* Read a line using buffering */
-	/* If last line is empty (in line mode) then it isn't outputed */
+/*
+  Read a line using buffering
+  If last line is empty (in line mode) then it isn't outputed
+*/
 
 
 READ_INFO::READ_INFO(File file_par, uint tot_length, String &field_term,
@@ -489,6 +608,22 @@ READ_INFO::READ_INFO(File file_par, uint tot_length, String &field_term,
       my_free((gptr) buffer,MYF(0)); /* purecov: inspected */
       error=1;
     }
+    else 
+    {
+      /*
+	init_io_cache() will not initialize read_function member
+	if the cache is READ_NET. The reason is explained in
+	mysys/mf_iocache.c. So we work around the problem with a
+	manual assignment
+      */
+      if (get_it_from_net)
+	cache.read_function = _my_b_net_read;
+
+      need_end_io_cache = 1;
+      if (!opt_old_rpl_compat && mysql_bin_log.is_open())
+	cache.pre_read = cache.pre_close =
+	  (IO_CACHE_CALLBACK) log_loaded_block;
+    }
   }
 }
 
@@ -497,7 +632,8 @@ READ_INFO::~READ_INFO()
 {
   if (!error)
   {
-    end_io_cache(&cache);
+    if (need_end_io_cache)
+      ::end_io_cache(&cache);
     my_free((gptr) buffer,MYF(0));
     error=1;
   }
@@ -537,10 +673,10 @@ int READ_INFO::read_field()
   if (found_end_of_line)
     return 1;					// One have to call next_line
 
-  /* Skipp until we find 'line_start' */
+  /* Skip until we find 'line_start' */
 
   if (start_of_line)
-  {						// Skipp until line_start
+  {						// Skip until line_start
     start_of_line=0;
     if (find_start_of_fields())
       return 1;
@@ -644,9 +780,13 @@ int READ_INFO::read_field()
 	  row_end=  to;
 	  return 0;
 	}
-	/* Copy the found '"' character */
+	/*
+	  The string didn't terminate yet.
+	  Store back next character for the loop
+	*/
 	PUSH(chr);
-	chr='"';
+	/* copy the found term character to 'to' */
+	chr= found_enclosed_char;
       }
       else if (chr == field_term_char && found_enclosed_char == INT_MAX)
       {
@@ -683,7 +823,7 @@ found_eof:
 /*
 ** One can't use fixed length with multi-byte charset **
 */
- 
+
 int READ_INFO::read_fixed_length()
 {
   int chr;
@@ -692,7 +832,7 @@ int READ_INFO::read_fixed_length()
     return 1;					// One have to call next_line
 
   if (start_of_line)
-  {						// Skipp until line_start
+  {						// Skip until line_start
     start_of_line=0;
     if (find_start_of_fields())
       return 1;
